class HttpException(Exception):
    """base exception for http request"""
    def __init__(self, message=None, details=None, status_code=None):
        self.message = self.__class__.__name__ if message is None else message
        super(Exception, self).__init__(self.message)
        self.details = details
        self.status_code = status_code

    def __unicode__(self):
        msg = self.__class__.__name__ + ": " + self.message
        if self.details:
            msg += ", " + self.details
        return msg

    def __str__(self):
        return self.__unicode__()


class NotFoundException(HttpException):
    """404"""
    def __init__(self, message=None, details=None, status_code=None):
        super(NotFoundException, self).__init__(message, details, status_code)


class InvalidResponse(HttpException):
    """response from server is not valid for this request."""
    def __init__(self, response):
        super(InvalidResponse, self).__init__()
        self.response = response



# -*- encoding: utf-8 -*-
import mock
import six

from opensearchsdk.utils import prepare_url
from opensearchsdk.tests import base


COMMON_PARAMS = {'Version': 'v2',
                 'AccessKeyId': 'testid',
                 'SignatureMethod': 'HMAC-SHA1',
                 'SignatureVersion': '1.0',
                 'SignatureNonce': '14053016951271226',
                 'Timestamp': '2014-07-14T01:34:55Z'
                 }
BODY = {'query': "config=format:json,start:0,hit:20&&query:'的'",
        'index_name': 'ut_3885312',
        'format': 'json',
        'fetch_fields': 'title;gmt_modified'}
METHOD = 'GET'
KEY_ID = 'testid'
KEY = 'testsecret'
SIGNATURE = "fxGidmIYSsx2AMa8onxuavOijuE="
STEP_1 = 'AccessKeyId=testid&SignatureMethod=HMAC-SHA1&SignatureNonce=1405' \
         '3016951271226&SignatureVersion=1.0&Timestamp=2014-07-14T01%3A34%' \
         '3A55Z&Version=v2&fetch_fields=title%3Bgmt_modified&format=json&i' \
         'ndex_name=ut_3885312&query=config%3Dformat%3Ajson%2Cstart%3A0%2C' \
         'hit%3A20%26%26query%3A%27%E7%9A%84%27'
STEP_2 = "GET&%2F&AccessKeyId%3Dtestid%26SignatureMethod%3DHMAC-SHA1%26Signatu" \
         "reNonce%3D14053016951271226%26SignatureVersion%3D1.0%26Timestamp%3D2" \
         "014-07-14T01%253A34%253A55Z%26Version%3Dv2%26fetch_fields%3Dtitle%25" \
         "3Bgmt_modified%26format%3Djson%26index_name%3Dut_3885312%26query%3Dc" \
         "onfig%253Dformat%253Ajson%252Cstart%253A0%252Chit%253A20%2526%2526qu" \
         "ery%253A%2527%25E7%259A%2584%2527"
SAFE = {' ': '%20', '~': '~'}


class TokenTest(base.TestCase):

    def test_get_common_params(self):
        common_params = prepare_url.get_common_params(KEY_ID)
        self.assertEqual(6, len(common_params))
        self.assertEqual(COMMON_PARAMS['Version'], common_params['Version'])
        self.assertEqual(
            COMMON_PARAMS['AccessKeyId'], common_params['AccessKeyId'])
        self.assertEqual(
            COMMON_PARAMS['SignatureMethod'], common_params['SignatureMethod'])
        self.assertEqual(
            COMMON_PARAMS['SignatureVersion'],
            common_params['SignatureVersion'])
        self.assertEqual(20, len(common_params['Timestamp']))

    def test_url_quote(self):
        for k, v in SAFE.items():
            quoted = prepare_url.url_quote(k)
            self.assertEqual(v, quoted)

    def test_get_quote_body(self):
        body = BODY
        body.update(COMMON_PARAMS)
        quoted_body = prepare_url.get_quote_body(body)
        self.assertEqual(STEP_1, quoted_body)

    def test_get_str_to_sign(self):
        str_to_sign = prepare_url.get_str_to_sign(STEP_1, METHOD)
        self.assertEqual(STEP_2, str_to_sign)

    def test_sign_str(self):
        signature = prepare_url.sign_str(KEY, STEP_2)
        if isinstance(signature, six.binary_type):
            signature = signature.decode()
        self.assertEqual(SIGNATURE, signature)

    @mock.patch('opensearchsdk.utils.prepare_url.get_common_params',
                mock.Mock(return_value=COMMON_PARAMS))
    def test_get_signature(self):
        signature = prepare_url.get_signature(METHOD, BODY, KEY, KEY_ID)
        if isinstance(signature, six.binary_type):
            signature = signature.decode()
        self.assertEqual(SIGNATURE, signature)



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.log import LogManager


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.log_manager = LogManager('', '')
        mock_send = mock.Mock()
        Manager.send_get = Manager.send_post = mock_send

    def test_get(self):
        self.log_manager.get('a', '1', 1, 'ASC')
        body = dict(page='1',
                    page_size='1',
                    sort_mode='ASC')
        spec_url = '/a'
        Manager.send_post.assert_called_with(body, spec_url)



import requests
import unittest


class TestCase(unittest.TestCase):
    """wrap testtools.TestCase"""
    def setUp(self):
        super(TestCase, self).setUp()


class TestResponse(requests.Response):
    """wrap request.Response"""
    def __init__(self, data):
        super(TestResponse, self).__init__()
        self._text = None
        self.status_code = data.get('status_code')
        self.headers = data.get('headers')
        self._text = data.get('text')
        self.raise_e = data.get('raise_e', '')
    #
    # def __eq__(self, other):
    #     return self.__dict__ == other.__dict__

    def json(self):
        if self.raise_e:
            raise self.raise_e
        return self._text



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class SuggestManager(api_base.Manager):
    """Suggest resource manage class"""
    def suggest(self, query, index_name, suggest_name, hit=10):
        """
        get suggest from user input.
        :param query: query string
        :param index_name: application name
        :param suggest_name: suggestion rule
        :param hit: count of suggestion, 1-10, default 10
        :return:
        """
        body = dict(query=query,
                    index_name=index_name,
                    suggest_name=suggest_name,
                    hit=str(hit))
        return self.send_get(body)



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class IndexManager(api_base.Manager):
    """Index resource manage class"""
    def refactor(self, name, operate=None, table_name=None):
        """
        refactor index
        :param name: application name
        :param operate: weather import data for refactor
        :param table_name: import data tables name, separated by ','
        :return:
        """
        body = dict(action='createtask')
        if operate and table_name:
            body['operate'] = 'import'
            body['table_name'] = table_name
        spec_url = '/' + name

        return self.send_get(body, spec_url)



import mock
import requests

from opensearchsdk.apiclient import api_base
from opensearchsdk.apiclient import exceptions
from opensearchsdk.tests import base
from opensearchsdk.utils import prepare_url


URL = 'http://www.aliyun.com'
RESOURCE_URL = '/index'
SPEC_URL = ''
HEADERS = {'User-Agent': 'ali-opensearch-python-client',
           'Content-Type': 'application/x-www-form-urlencoded'}
BODY = {'a': '1'}
SIGN = 'sign'
SIGNED_BODY = {'Signature': SIGN}
SIGNED_BODY.update(BODY)
ENCODE_BODY = "{'b': 2}"
KEY = 'key'
KEY_ID = 'key_id'
GET = 'GET'
POST = 'POST'
FAKE_RESP = {
    "status_code": 200,
    "text": BODY}
FAKE_INVALID_RESP = {
    "status_code": 200,
    "text": BODY,
    "raise_e": ValueError}
RESP_404 = {
    "status_code": 404,
    "text": BODY,
    "raise_e": ValueError}
RESP_400 = {
    "status_code": 400,
    "text": BODY}


class FakeClient(object):
    key = KEY
    key_id = KEY_ID
    http_client = api_base.HTTPClient(URL)


class HTTPClientTest(base.TestCase):
    def setUp(self):
        super(HTTPClientTest, self).setUp()
        requests.request = mock.Mock(
            return_value=base.TestResponse(FAKE_RESP))
        self.http_client = api_base.HTTPClient(URL)

    def test_request_get(self):
        resp = self.http_client.request(GET, RESOURCE_URL)
        requests.request.assert_called_with(
            GET, URL + RESOURCE_URL, headers=HEADERS)
        self.assertEqual(BODY, resp)

    def test_request_post(self):
        resp = self.http_client.request(POST, RESOURCE_URL)
        requests.request.assert_called_with(
            POST, URL + RESOURCE_URL, headers=HEADERS)
        self.assertEqual(BODY, resp)

    @mock.patch.object(requests, 'request',
                       return_value=base.TestResponse(FAKE_INVALID_RESP))
    def test_invalid_response(self, mock_req):
        self.assertRaises(exceptions.InvalidResponse,
                          self.http_client.request, POST, RESOURCE_URL)

    @mock.patch.object(requests, 'request',
                       return_value=base.TestResponse(RESP_404))
    def test_request_404(self, mock_req):
        self.assertEqual('NotFoundException: NotFoundException, app',
                         str(exceptions.NotFoundException(details='app')))
        self.assertRaises(exceptions.NotFoundException,
                          self.http_client.request, POST, RESOURCE_URL)

    @mock.patch.object(requests, 'request',
                       return_value=base.TestResponse(RESP_400))
    def test_request_400(self, mock_req):
        self.assertRaises(exceptions.HttpException,
                          self.http_client.request, POST, RESOURCE_URL)


class ManagerTest(base.TestCase):
    def setUp(self):
        super(ManagerTest, self).setUp()
        self.ori_request = api_base.HTTPClient.request
        self.ori_get_signature = prepare_url.get_signature
        api_base.HTTPClient.request = mock.Mock(return_value=FAKE_RESP)
        prepare_url.get_signature = mock.Mock(return_value=SIGN)
        self.manager = api_base.Manager(FakeClient, RESOURCE_URL)

    def tearDown(self):
        super(ManagerTest, self).tearDown()
        api_base.HTTPClient.request = self.ori_request
        prepare_url.get_signature = self.ori_get_signature

    def test_request(self):
        resp = self.manager.send_request(GET, SPEC_URL, BODY)
        api_base.HTTPClient.request.assert_called_with(
            GET, RESOURCE_URL+SPEC_URL, data=SIGNED_BODY)
        prepare_url.get_signature.assert_called_with(GET, BODY, KEY, KEY_ID)
        self.assertEqual(FAKE_RESP, resp)

    @mock.patch('opensearchsdk.apiclient.api_base.urlencode')
    def test_get(self, mock_urlencode):
        mock_urlencode.return_value = ENCODE_BODY
        resp = self.manager.send_get(BODY, SPEC_URL)

        prepare_url.get_signature.assert_called_with(GET, BODY, KEY, KEY_ID)
        mock_urlencode.assert_called_with(BODY)
        self.assertEqual(FAKE_RESP, resp)

    @mock.patch.object(api_base.Manager, 'send_request')
    def test_post(self, mock_request):
        mock_request.return_value = FAKE_RESP
        resp = self.manager.send_post(BODY, SPEC_URL)

        mock_request.assert_called_with(POST, SPEC_URL, BODY)
        self.assertEqual(FAKE_RESP, resp)



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class DataManager(api_base.Manager):
    """Data Process resource manage class"""
    def create(self, app_name, table_name, items):
        """
        create data process to application
        :param app_name: app name
        :param table_name: table name of upload items
        :param items: items in json format
        :return:{"status":"OK","request_id":"10373587"}
        """
        body = dict(table_name=table_name, items=items)
        spec_url = '/' + app_name
        return self.send_post(body, spec_url)



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class SearchManager(api_base.Manager):
    """Search resource manage class"""
    def search(self, query=None, index_name=None, fetch_fields=None, qp=None,
               disable=None, first_formula_name=None, formula_name=None,
               summary=None, scroll=None, search_type=None, scroll_id=None):
        """
        do search with given parameters
        :param query: query string
        :param index_name: application name(s), separate by ';'
        :param fetch_fields: field to return, separate by ';'
        :param qp: search analyse rules, separate by ','
        :param disable: whether turn off search analyse
        :param first_formula_name:
        :param formula_name:
        :param summary:
        :param scroll: expire time, default ms
        :param search_type: scan
        :param scroll_id: last search id, None if first time search
        :return: dict, search result
        """
        body = {}

        def _simple_search():
            body['index_name'] = index_name
            body['query'] = query
            if fetch_fields:
                body['fetch_fields'] = fetch_fields
            if qp:
                body['qp'] = qp
            if disable:
                body['disable'] = disable
            if first_formula_name:
                body['first_formula_name'] = first_formula_name
            if formula_name:
                body['formula_name'] = formula_name
            if summary:
                body['summary'] = summary

        # check whether do combine search
        if scroll:
            body['scroll'] = scroll

            # if not the first time, must with last search id.
            if scroll_id:
                body['scroll_id'] = scroll_id
            else:
                # first time do combine search
                body['search_type'] = search_type
                _simple_search()
        else:
            _simple_search()

        return self.send_get(body)



# -*- encoding: utf-8 -*-
import logging
import requests
from six.moves.urllib.parse import urlencode
from opensearchsdk.apiclient import exceptions
from opensearchsdk.utils import prepare_url


USER_AGENT = 'ali-opensearch-python-client'
_logger = logging.getLogger(__name__)


class HTTPClient(object):
    """HTTP client for sending request to server"""
    def __init__(self, base_url):
        self.base_url = base_url

    def request(self, method, url, **kwargs):
        url = self.base_url + url
        kwargs.setdefault("headers", {})
        kwargs["headers"]["User-Agent"] = USER_AGENT
        kwargs["headers"]["Content-Type"] = 'application/x-www-form-urlencoded'
        # self._logger_req(method, url, **kwargs)
        resp = requests.request(method, url, **kwargs)
        # self._logger_resp(resp)
        try:
            resp.raise_for_status()
        except requests.RequestException as e:
            if resp.status_code == 404:
                exc_type = exceptions.NotFoundException
            else:
                exc_type = exceptions.HttpException
            raise exc_type(e,
                           details=self._parse_error_resp(resp),
                           status_code=resp.status_code)
        try:
            resp_body = resp.json()
        except ValueError:
            raise exceptions.InvalidResponse(response=resp)
        return resp_body

    def _parse_error_resp(self, resp):
        try:
            jresp = resp.json()
            return jresp
        except ValueError:
            pass
        return resp.text
    #
    # def _logger_req(self, method, url, **kwargs):
    #     if not _logger.isEnabledFor(logging.DEBUG):
    #         return
    #     string_parts = [
    #         "curl -i",
    #         "-X '%s'" % method,
    #         "'%s'" % url,
    #     ]
    #     if method != 'GET':
    #         for element in kwargs['headers'].items():
    #             header = " -H '%s: %s'" % element
    #             string_parts.append(header)
    #         string_parts.append('--data')
    #         data = kwargs.get('data', '')
    #         if data:
    #             data_str = urllib.urlencode(data)
    #             data_str = "'" + data_str + "'"
    #             string_parts.append(data_str)
    #     print("REQ: %s" % " ".join(string_parts))
    #     _logger.debug("REQ: %s" % " ".join(string_parts))
    #
    # def _logger_resp(self, resp):
    #     if not _logger.isEnabledFor(logging.DEBUG):
    #         return
    #     _logger.debug(
    #         "RESP: [%s] %r" % (
    #             resp.status_code,
    #             resp.headers,
    #         ),
    #     )
    #     # if resp._content_consumed:
    #     #     _logger.debug(
    #     #         "RESP BODY: %s",
    #     #         resp.text,
    #     #     )
    #     _logger.debug(
    #         "encoding: %s",
    #         resp.encoding,
    #     )


class Manager(object):
    def __init__(self, api, resource_url):
        self.api = api
        self.resource_url = resource_url

    def send_request(self, method, spec_url, body):
        key = self.api.key
        key_id = self.api.key_id
        body['Signature'] = prepare_url.get_signature(
            method, body, key, key_id)
        final_url = self.resource_url + spec_url
        return self.api.http_client.request(method, final_url, data=body)

    def send_get(self, body, spec_url=''):
        key = self.api.key
        key_id = self.api.key_id
        body['Signature'] = prepare_url.get_signature(
            'GET', body, key, key_id)
        encoded_url = urlencode(body)
        final_url = self.resource_url + spec_url + '?' + encoded_url
        return self.api.http_client.request('GET', final_url)

    def send_post(self, body, spec_url=''):
        return self.send_request('POST', spec_url, body)



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.index import IndexManager


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.index_manager = IndexManager('', '')
        mock_send = mock.Mock()
        Manager.send_get = Manager.send_post = mock_send

    def test_refactor_without_data(self):
        self.index_manager.refactor('a')
        body = dict(action='createtask',)
        spec_url = '/a'
        Manager.send_post.assert_called_with(body, spec_url)

    def test_refactor_with_data(self):
        self.index_manager.refactor('a', 'b', 'c')
        body = dict(action='createtask',
                    operate='import',
                    table_name='c')
        spec_url = '/a'
        Manager.send_post.assert_called_with(body, spec_url)



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.data import DataManager


FAKE_RESP = {'data': 'name'}


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.data_manager = DataManager('', '')
        mock_send = mock.Mock(return_value=FAKE_RESP)
        Manager.send_get = Manager.send_post = mock_send

    def test_list(self):
        resp = self.data_manager.create('a', '1', '2')
        self.assertEqual(FAKE_RESP, resp)
        Manager.send_post.assert_called_with({'table_name': '1',
                                              'items': '2'},
                                             '/a')



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.suggest import SuggestManager


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.suggest_manager = SuggestManager('', '')
        mock_send = mock.Mock()
        Manager.send_get = Manager.send_post = mock_send

    def test_list(self):
        self.suggest_manager.suggest('a', 'b', 'c', 1)
        body = dict(query='a',
                    index_name='b',
                    suggest_name='c',
                    hit='1')
        Manager.send_post.assert_called_with(body)



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.search import SearchManager


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.search_manager = SearchManager('', '')
        mock_send = mock.Mock()
        Manager.send_get = Manager.send_post = mock_send

    def test_search(self):
        # simple search
        self.search_manager.search('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h')
        body = dict(query='a', index_name='b', fetch_fields='c', qp='d',
                    disable='e', first_formula_name='f', formula_name='g',
                    summary='h')
        Manager.send_get.assert_called_with(body)

        # first combine search
        body['scroll'] = '1h'
        body['search_type'] = 'scan'

        self.search_manager.search('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h',
                                   '1h', 'scan')
        Manager.send_get.assert_called_with(body)

        # second combine search
        body = dict(scroll='1h', scroll_id='i')
        self.search_manager.search(**body)
        Manager.send_get.assert_called_with(body)



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class AppManager(api_base.Manager):
    """Application resource manage class"""
    def list(self, page=None, page_size=None):
        """
        get application list with given page and page size.
        :param page:
        :param page_size:
        :return: app list
        """
        body = {}
        if page is not None:
            body['page'] = str(page)
        if page_size is not None:
            body['page_size'] = str(page_size)

        return self.send_get(body)

    def create(self, name, template):
        """
        create a application from a given template.
        :param name: application name
        :param template: template name
        :return:application name
        """
        body = dict(action='create', template=template)
        spec_url = '/' + name

        return self.send_post(body, spec_url)

    def delete(self, name):
        """
        delete a application by name.
        :param name: application name
        :return: status messsage
        """
        body = dict(action='delete')
        spec_url = '/' + name

        return self.send_post(body, spec_url)

    def get(self, name):
        """
        get detail of a application by name.
        :param name: application name
        :return: application detail
        """
        body = dict(action='status')
        spec_url = '/' + name

        return self.send_post(body, spec_url)



from opensearchsdk import client
from opensearchsdk.tests import base


URL = 'http://www.aliyun.com'
KEY = 'KEY'
KEY_ID = 'KEY_ID'


class ClientTest(base.TestCase):
    def test_passing_property(self):
        c = client.Client(URL, KEY, KEY_ID)
        self.assertEqual(KEY, c.key)
        self.assertEqual(URL, c.http_client.base_url)
        self.assertEqual(client.APP_URL, c.app.resource_url)
        self.assertEqual(c, c.app.api)



# -*- coding:utf-8 -*-
import base64
import hashlib
import hmac
import six
import sys
import time
import uuid
try:
    from urllib import quote
except ImportError:
    from urllib.parse import quote


def url_quote(encode_str):
    if isinstance(encode_str, six.binary_type):
        if sys.stdin.encoding is None:
            encode_str = encode_str.decode('cp936').encode('utf8')
        else:
            encode_str = encode_str.decode(sys.stdin.encoding).encode('utf8')
    res = quote(encode_str, '~')
    res = res.replace('+', '%20')
    res = res.replace('*', '%2A')
    return res


def get_common_params(key_id):
    params = dict(Version="v2",
                  SignatureMethod="HMAC-SHA1",
                  SignatureVersion="1.0",
                  Timestamp=time.strftime(
                      "%Y-%m-%dT%H:%M:%SZ", time.gmtime()),
                  SignatureNonce=str(uuid.uuid4()),
                  AccessKeyId=key_id
                  )
    return params


def get_quote_body(body):
    items = body.items()
    items = sorted(items, key=lambda x: x[0])
    encode_str = ''
    for k, v in items:
        encode_str += url_quote(k) + '=' + url_quote(v) + '&'
    encode_str = encode_str[:-1]
    return encode_str


def get_str_to_sign(encode_str, method):
    encode_str = url_quote(encode_str)
    to_sign = method + '&%2F&' + encode_str
    return to_sign


def sign_str(key, str_to_sign):
    key += '&'
    if isinstance(str_to_sign, six.text_type):
        str_to_sign = str_to_sign.encode()
    if isinstance(key, six.text_type):
        key = key.encode()
    sign = hmac.new(key, str_to_sign, hashlib.sha1).digest()
    signature = base64.encodestring(sign).strip()
    return signature


def get_signature(method, body, key, key_id):
    common_params = get_common_params(key_id)
    body.update(common_params)
    quote_body = get_quote_body(body)
    str_to_sign = get_str_to_sign(quote_body, method)
    signature = sign_str(key, str_to_sign)
    return signature



# -*- encoding: utf-8 -*-
from opensearchsdk.apiclient import api_base


class LogManager(api_base.Manager):
    """Error Log resource manage class"""
    def get(self, name, page, page_size, sort_mode):
        """
        get error log of application
        :param name: application name
        :param page: (str)page number
        :param page_size: (int)page size
        :param sort_mode: 'ASC' or 'DESC'
        :return: (json)log content
        """
        body = dict(page=page,
                    page_size=str(page_size),
                    sort_mode=sort_mode)
        spec_url = '/' + name

        return self.send_get(body, spec_url)



# -*- encoding: utf-8 -*-
"""
opensearch python sdk client.
"""
from opensearchsdk.apiclient import api_base
from opensearchsdk.v2 import app
from opensearchsdk.v2 import data
from opensearchsdk.v2 import search
from opensearchsdk.v2 import suggest
from opensearchsdk.v2 import index
from opensearchsdk.v2 import quota
from opensearchsdk.v2 import log


APP_URL = '/index'
DATA_URL = APP_URL + '/doc'
SEARCH_URL = '/search'
INDEX_URL = '/index'
SUGGEST_URL = '/suggest'
QUOTA_URL = ''
# TODO(Yan Haifeng) not yet implement
LOG_URL = APP_URL + '/error'


class Client(object):
    """opensearch python sdk client."""

    def __init__(self, base_url, key, key_id):
        self.base_url = base_url
        self.key = key
        self.key_id = key_id
        self.http_client = api_base.HTTPClient(base_url)
        self.app = app.AppManager(self, APP_URL)
        self.data = data.DataManager(self, DATA_URL)
        self.search = search.SearchManager(self, SEARCH_URL)
        self.suggest = suggest.SuggestManager(self, SUGGEST_URL)
        self.index = index.IndexManager(self, INDEX_URL)
        self.quota = quota.QuotaManager(self, QUOTA_URL)
        # TODO(Yan Haifeng) not yet implement
        self.log = log.LogManager(self, LOG_URL)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
from setuptools import setup, find_packages

setup(
    name='ali-opensearch',
    version='2.0.3',
    packages=find_packages(),
    keywords=('OpenSearch SDK', 'Ali Cloud'),
    description='Python SDK for OpenSearch of Ali Cloud',
    license='Apache License Version 2.0',
    url='https://github.com/yanheven/ali-opensearch-sdk',
    author='Yan Haifeng(颜海峰)',
    author_email='yanheven@qq.com',

    include_package_data=True,
    platforms='any',
    install_requires=['requests', 'six'],
    classifiers=['Operating System :: MacOS',
                 'Operating System :: Microsoft :: Windows',
                 'Operating System :: POSIX :: Linux',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3.5']
)



# -*- encoding: utf-8 -*-
import json

from opensearchsdk.client import Client


def list_app(client):
    apps = client.app.list(page=1, page_size=1)
    print(apps)
    '''
    {
        u'status': u'OK',
        u'total': u'1',
        u'result': [
                        {
                            u'description': u'test',
                            u'created': u'1447753400',
                            u'id': u'119631',
                            u'name': u'test'
                        }
                    ],
        u'request_id': u'1447866418002866600607068'
    }
    '''


def create_app(client):
    app = client.app.create('test_app', 'a')
    print(app)
    '''
    {
        u'status': u'OK',
        u'result': {
                        u'index_name': u'test_app'
                    },
        u'request_id': u'1449146731021309800470027'
    }

    '''


def creat_data_process(client):
    table_name = 'test_table'
    items = [
        {
            "cmd": "add",
            "timestamp": 1401342874777,
            "fields": {
                "id": "1",
                "title": "This is the title",
                "body": "This is the body"
            }
        },
        {
            "cmd": "update",
            "timestamp": 1401342874778,
            "fields": {
                "id": "2",
                "title": "This is the new title"
            }
        },
        {
            "cmd": "delete",
            "fields": {
                "id": "3"
            }
        }
    ]
    items = json.dumps(items)
    data_ret = client.data.create('test_app', table_name, items)
    print(data_ret)
    '''
        {
            u'status': u'OK',
            u'total': u'1',
            u'result': [
                            {
                                u'description': u'test',
                                u'created': u'1447753400',
                                u'id': u'119631',
                                u'name': u'test'
                            }
                        ],
            u'request_id': u'1448079259020634800692900'
        }
    '''


def search(client):
    # simple search
    body = dict(index_name='test_app',
                query='query=id:"0"',
                fetch_fields='testname',
                first_formula_name='default',
                formula_name='default',
                summary='summary_snipped:1,summary_field:title,summ'
                        'ary_element:high,summary_len:32,summary_el'
                        'lipsis:...;summary_snipped:2,summary_field'
                        ':body,summary_element:high,summary_len:60,s'
                        'ummary_ellipsis:...')

    def simple_search():
        res = client.search.search(**body)
        print(res)
        '''
        {
            u'status': u'OK',
            u'tracer': u'',
            u'errors': [],
            u'result': {
                            u'searchtime': 0.005986,
                            u'items': [
                                        {
                                            u'index_name': u'test_app',
                                            u'testname': u'0',
                                        }
                                    ],
                            u'facet': [],
                            u'viewtotal': 1,
                            u'num': 1,
                            u'total': 1
                        },
            u'request_id': u'144932743917790790023680'
        }
        '''

    def combine_search_1():
        # first combine search
        body['scroll'] = '1h'
        body['search_type'] = 'scan'
        res = client.search.search(**body)
        print(res)
        '''
        {
            u'status': u'OK',
            u'tracer': u'',
            u'errors': [],
            u'result': {
                            u'scroll_id': u'eJyNjk1LxDAQhn9NcizdZNnSQw+rXkTUk
                            +dQk2k7ki8zqW7315sWhRUVhAwZnnmHZ45aA9EdLLemexpz87
                            Ab8f7t0b2fw5kr21NWek4UUjdgoozeQyKdgrVouCqdVXCKKqO
                            Dbrfft1KKRtZt2/ABsp7UgGANdRko+94BR2/gpNZ2Y6qPkb/O
                            kJZOBz/gyOTNEJLrM5PHFwqeiUN5W6KM0BTMhKhLcZqd68vi5
                            6/IY4ywJnZMXH/R7YDCMmYLFxwsOPCrZsJxuhhY8AVK8S1rMR
                            JS4VVVMXn1Uyl+UT4Hs/zfeKj/NH4A34SVcg==',
                            u'searchtime': 0.002997,
                            u'items': [],
                            u'facet': [],
                            u'viewtotal': 1,
                            u'num': 0,
                            u'total': 1
                        },
            u'request_id': u'144932913017790788000125'
        }
        '''
        return res.get('result').get('scroll_id')

    def combine_search_2(scroll_id):
        del body['scroll']
        body['scroll_id'] = scroll_id
        res = client.search.search(**body)
        print(res)
        '''
        the same as simple search
        '''

    simple_search()
    scroll_id = combine_search_1()
    combine_search_2(scroll_id)


def suggest(client):
    res = client.suggest.suggest('的', 'test_app', 's2')
    print(res)
    '''
    {
        u'errors': [
            {
                u'message': u'nosuggesterdataonline',
                u'code': 2422
            }
        ],
        u'suggestions': [],
        u'searchtime': 0.034475,
        u'request_id': u'144937817217790789042028'
    }
    '''


def index_refactor(client):
    # only refactor index
    def refactor_only():
        res = client.index.refactor('test_app')
        print(res)
        '''
        {
            u'status': u'OK',
            u'result': {
                u'task_id': u'159469'
            },
            u'request_id': u'1449378626069143700426870'
        }
        '''
    def refactor_import_data():
        #  refactor index and import data
        res = client.index.refactor('test_app', 'import', 'id')
        print(res)

    # refactor_only()
    refactor_import_data()


def get_error_log(client):
    res = client.log.get('test_app', '1', 1, 'ASC')
    print(res)
    '''
    {
        u'status': u'OK',
        u'result': {
            u'count': 0,
            u'items': [],
            u'page': 1,
            u'page_size': u'1'
        },
        u'request_id': u'1449379073037249300677090'
    }
    '''


if __name__ == '__main__':
    import mykey
    import logging
    LOG = logging.basicConfig(level=logging.DEBUG,
                              format="[%(asctime)s] %(name)s:"
                                     "%(levelname)s: %(message)s")
    url = 'http://opensearch-cn-hangzhou.aliyuncs.com'
    key = mykey.KEY['key_secrete']
    key_id = mykey.KEY['key_id']
    client = Client(url, key, key_id)
    # list_app(client)
    # create_app(client)

    # must create app before create data process

    # creat_data_process(client)
    # search(client)
    # suggest(client)
    # index_refactor(client)
    # get_error_log(client)



import mock

from opensearchsdk.apiclient.api_base import Manager
from opensearchsdk.tests import base
from opensearchsdk.v2.app import AppManager


FAKE_RESP = {'app': 'name'}


class AppTest(base.TestCase):
    def setUp(self):
        super(AppTest, self).setUp()
        self.app_manager = AppManager('', '')
        mock_send = mock.Mock(return_value=FAKE_RESP)
        Manager.send_get = Manager.send_post = mock_send

    def test_list(self):
        resp = self.app_manager.list(1, 2)
        self.assertEqual(FAKE_RESP, resp)
        Manager.send_post.assert_called_with({'page': '1', 'page_size': '2'})

    def test_create(self):
        self.app_manager.create('a', 'b')
        Manager.send_post.assert_called_with({'action': 'create',
                                              'template': 'b'},
                                             '/a')

    def test_delete(self):
        self.app_manager.delete('a')
        Manager.send_post.assert_called_with({'action': 'delete'}, '/a')

    def test_get(self):
        self.app_manager.get('a')
        Manager.send_post.assert_called_with({'action': 'status'}, '/a')
